#define _GNU_SOURCE
#include <unistd.h>
#include <stdio.h>
#include <stdbool.h>
#include <string.h>
#include <errno.h>
#include <signal.h>
#include <assert.h>

#include <MX370x.nsmap> // this file must be included only once in the project

#include "utils.h"

/** @file minmax.c
 *
 * @brief This file is an exemple how one may perform an minmax acquisition on a MX3701 module using direct access.
 *
 * @note This software is not interactive. Please configure the sensible variables to fit your needs.
 *
 * Commands are sent to the module on top of the SOAP protocol.
 * We use the tool "gsoap" to generate C-code that allows to integrate this protocol transparently in applications.
 * All what developpers have to do is to use the provided API.
 * (This API is detailled in the file soap/doc/html/index.html in this distribution)
 *
 * __Difference between the API description and the actual function calls__
 *
 * The API documentation is accurate but doesn't decrive exactly the prototype of the functions.
 * Anyway transposing a documented function to the actual one is straightforward.
 *
 * Given a function YYY described in the API documentation
 * - the actual function to call are named soap_call_YYY
 * - every function has three more parameters, namelly
 *   - a pointer to a "struct soap" object, that must be initialized first
 * 	 - the SOAP endpoint: a string defining how to access the module. Usually this is a dotted IP address,
 *     but it may as well be a symbolic name ( as long as the DNS resolver knows it )
 *   - a third parameter that is always NULL on our case
 *
 *  Note that the second parameter needs to be given only to the first SOAP call.
 *  As the connection is already established afterwhile, NULL can be used then.
 *
 * This note applies only to function prototypes. Datatypes are exactly described in the API doc.
 */

/* BEGIN OF SOURCE CODE */

//-----------------------------------------------------------------------------------------
/* PARAMETERIZATION OF THE EXAMPLE  */
//-----------------------------------------------------------------------------------------

/* store a pointer to the IP address*/
char * ip = NULL;

//-----------------------------------------------------------------------------------------
/* please see utils.h */
int DUMMY = 0;

//-----------------------------------------------------------------------------------------
/** this is the parameter used to configure an acquisition
 * They are passed to the function soap_call_MX370x__TransducerInitAndStartAutoRefresh()
 *
 */
struct
{
  xsd__unsignedLong   ulTransducerSelection;
  xsd__unsignedLong   ulChannelMask;
  xsd__unsignedLong   ulDivisionFactor;
  xsd__unsignedLong   ulStopChannelMask;
  xsd__unsignedLong   ulStopCondition;
  xsd__unsignedLong   ulStopValue;
  xsd__unsignedLong   ulOption1;
  xsd__unsignedLong   ulOption2;
  xsd__unsignedLong   ulOption3;
  xsd__unsignedLong   ulOption4;
} params =
{
.ulTransducerSelection=3, /**< Transducer type selection */
.ulChannelMask=3, /**< Mask of the channel for the min/max evaluation (1 bit = 1 Channel) */
.ulDivisionFactor=5, /**< Division factor min : 5, max 255 */
.ulStopChannelMask=0, /**< Stop channel mask */
.ulStopCondition=0, /**<  0 : disabled - 1 : < - 2 : >  */
.ulStopValue=0, /* Stop value (24 bits) used for comparison */
.ulOption1=0, /**< Reserved */
.ulOption2=0, /**< Reserved */
.ulOption3=0, /**< Reserved */
.ulOption4=0, /**< Reserved */
};

//-----------------------------------------------------------------------------------------
/* END OF PARAMETERIZATION */
//-----------------------------------------------------------------------------------------

//-----------------------------------------------------------------------------------------
/** Translates error code returned by TransducerInitAndStartMinMaxAcquisition() to a string.
 *
 * @param[in] soapenv The current SOAP environment
 * @param[in] Response The parameter the
 *
 * @return a dynamically allocated string the function TransducerInitAndStartAutoRefresh() had received
 *
 * @note The string returned is dynamically malloc()ated and must be free()d after its use.
 */
static const char* get_TransducerInitAndStartMinMaxAcquisition_error_message(struct soap* soapenv, struct MX370x__Response* Response)
{
	switch(Response->iReturnValue)
	{
		case 0:
			return strdup("remote function performed OK");
		case -1:
			/* means an system error occured */
			{
				struct MXCommon__ByteArrayResponse Response1;
				assert_gsoap(soapenv,soap_call_MXCommon__Strerror(soapenv,NULL, NULL,Response->syserrno, &Response1) );
				return strdup(Response1.sArray.__ptr);
			}
		case -2:
			return strdup("Transducer selection error");
 		case -3:
 			return strdup("Channel mask can not be null");
 		case -4:
 		 	return strdup("channel mask error");
 		case -5:
 			return strdup("Division factor error");
 		case -6:
 		 	return strdup("Stop condition selection error");
 		case -7:
 		 	return strdup("Stop channel mask can not be null");
 		case -8:
 		 	return strdup("Stop channel mask error");
 		case -9:
 			return strdup("Stop value error");
 		case -100:
 			return strdup("TransducerInit kernel function error");
 		case -101:
 			return strdup("InitConvertTimeDivisionFactor kernel function error");
 		case -102:
 			return strdup("InitMinMaxAcquisition kernel function error");
 		case -103:
 			return strdup("StartStopMinMaxAcquisition kernel function error");
		default:
			return(strdup("unknown error code"));
	}
}
//-----------------------------------------------------------------------------------------
/** signal handler : stops the current acquisition and cleanup the SOAP connection.
 *
 */
static void do_signal (int sig)
{

	printf("signal received:%s\n",strsignal(sig));
	printf("stopping current acquisition\n");
	/* note: the soapenv variable should have been correctly initialised before the signal handler was set */
	{
		struct soap soapenv;
		mx_soap_init(&soapenv);
		char endpoint[30];
		/* generates the endpoint string
		 * SOAP "endpoints" are of the form NAME:PORT -
		 * PORT is in the case of the SOAP command server: 5555
		 */
		if ( snprintf(endpoint,sizeof(endpoint),"%s:5555",ip) < 0)
		{
			fprintf(stderr,"error: %s:%d!\n",__FUNCTION__,__LINE__);
			exit(1);
		}
		struct MX370x__Response Response;
		if ( ! ( SOAP_OK==soap_call_MX370x__TransducerStopAndReleaseMinMaxAcquisition(&soapenv, endpoint, NULL, &DUMMY, &Response) ) )
		{
			fprintf(stderr,"error while stopping acquisition: ");
			soap_print_fault(&soapenv, stderr);
		}
		mx_soap_clean(&soapenv);
	}

	exit(1);
}

//-----------------------------------------------------------------------------------------
/** For all signals set the handler do_signal.
 *
 * This allows to handle CTRL+C nicely
 *
 * @note This function must be called after the first call to a function of the soap_call() familly.
 */
static void setup_signal(void)
{
	struct sigaction action;
	/* catch all signal */
	sigfillset(&action.sa_mask); /* all other signal will be blocked when handling the one received */
	action.sa_handler = do_signal;
	action.sa_flags = 0;
	{
		int i;
		for (i=1; i< NSIG; i++)
		{
			if (SIGKILL==i) /* this signal can not be blocked */
				continue;
			if (SIGSTOP==i) /* this signal can not be blocked */
				continue;
			sigaction(i,&action,NULL); /* TODO: manage error */
		}
	}
}

//-----------------------------------------------------------------------------------------

/**
 * Function called when the parameters of the sample are wrong
 * The sample need the IP address of the system as parameter
 */
static void usage()
{
	fprintf(stderr, "Usage: ./autorefresh IP_ADDRESS TransducerIndex \n");
	fprintf(stderr, "TransducerIndex: Index of the transducer you want to use (see retrieveinfo sample -> Selection Index).\n");
	exit(1);
}


//-----------------------------------------------------------------------------------------
int main(int argc, char * argv[])
{

	/* this variable contains the current SOAP connection - */
	struct soap soapenv;
	char endpoint[30];
	int clientfd = -1;
	unsigned int number_of_channels = 0;

	/* check the parameters */
	if ( argc != 3 )
		usage();

	/* store a pointer to the IP */
	ip = argv[1];

	/* get the transducer index */
	params.ulTransducerSelection = atol(argv[2]);

	/* data initialisation */
	memset(endpoint	,0,sizeof(endpoint));
	mx_soap_init(&soapenv);

	/* generates the endpoint string
	 * SOAP "endpoints" are of the form NAME:PORT -
	 * PORT is in the case of the SOAP command server: 5555
	 */
	if ( snprintf(endpoint,sizeof(endpoint),"%s:5555",argv[1]) < 0)
	{
		fprintf(stderr,"error: %s:%d!\n",__FUNCTION__,__LINE__);
		exit(1);
	}

	/* connect to data server (if no acquisition was started, we should not receive any data) */
	clientfd = datasrv_connect(argv[1]);
	/* retrieve the number of channels on the module (4,8 or 16)
	 *
	 * @note On upcoming version of the firmware there will be a SOAP function dedicated to
	 * this task.
	 *
	 */
	{
		struct MXCommon__ByteArrayResponse response;

		assert_gsoap(&soapenv, soap_call_MXCommon__GetModuleType(&soapenv,endpoint , NULL, &DUMMY,&response) );
		/* extract the number of channels from the "type" string */
		assert( ! parse_number_of_channels (response.sArray.__ptr, &number_of_channels) );
	}

	/* send request to start the acquisition, using parameters set in the "params" global variable. */
	{
		struct MX370x__Response Response; // this is a general type for communicating with the MX module
		assert_gsoap(	&soapenv,
						soap_call_MX370x__TransducerInitAndStartMinMaxAcquisition(
															&soapenv,
															endpoint,
															NULL,
															params.ulTransducerSelection,
															params.ulChannelMask,
															params.ulDivisionFactor,
															params.ulStopChannelMask,
															params.ulStopCondition,
															params.ulStopValue,
															params.ulOption1,
															params.ulOption2,
															params.ulOption3,
															params.ulOption4,
															&Response) );
		/* manage errors */
		if ( Response.iReturnValue )
		{
			char * msg = get_TransducerInitAndStartMinMaxAcquisition_error_message(&soapenv,&Response);
			fprintf(stderr,"error at %s:%d\nmessage: %s\n", __FUNCTION__,__LINE__,msg);
			if (msg) free(msg);
			exit(1);
		}
	}

  	/* from now on, we want the acquisition to be stopped on program termination */
  	setup_signal();
  	/* now we should use abort() instead of exit() */
  	for(;;)
  	{
  		struct MX370x__TransducerGetMinMaxStatusResponse Response;
  		assert_gsoap(&soapenv, soap_call_MX370x__TransducerGetMinMaxStatus(&soapenv,NULL,NULL,&DUMMY,&Response));

		/* Release the memory allocated by the gSOAP calls */
		soap_destroy(&soapenv);
		soap_end(&soapenv);

  		if (Response.iReturnValue)
  		{
  			fprintf(stderr,"%s:%s:%d: remote error: %d\n",__FILE__,__FUNCTION__,__LINE__,Response.iReturnValue);
  			abort();
  		}

		/* did a PLD overflow happened ? */
		if (Response.ulOverFlow == 1)
		{
			printf("PLD Overflow !\n");
			break;
		}
		if (0==Response.ulFlag)
		{
			/* Disable */
			fprintf(stderr,"state is \"disable\" which should not be the case !\n");
			abort();
		}

		if (Response.ulFlag>2)
		{
			fprintf(stderr,"%s:%s:%d: impossible value for ulFlag: %lu\n",__FILE__,__FUNCTION__,__LINE__,Response.ulFlag);
			abort();
		}

		/* we have filtered all error cases - values are correct */
		{
			int i;
			for (i=0;i<number_of_channels;i++)
			{
	            if (params.ulChannelMask & (1<<i))
				{
	            	printf(" Channel %d Min:%lX Max:%lX\n", i,Response.pulMinValues[i],Response.pulMaxValues[i]);
				}
			}
		}

		if (1==Response.ulFlag)
		{
			/* in progress */
		}
		else
		if (2==Response.ulFlag) 		/* did we reached the compare value ? */
		{
			printf("\nCompare value reached !\n");
			break;
		}


  	}

	printf("OK\n");
	return 0;
}
