#ifndef __MX_UTILS_H__
#define __MX_UTILS_H__

int getch (void);
int getche(void);
int kbhit (void);

/** Some SOAP function have a parameter of type void*_
 *
 * These functions MUST receive a valid pointer ( not NULL ) but this address is not used.
 * Hence the DUMMY variable, that must be defined in a C body, preferably in main.c
 *
 * */
extern int DUMMY;

//-----------------------------------------------------------------------------------------
/** performs an orderly cleanup of the current SOAP state.
 *
 * Deletes mmemory allocated internally by the gSOAP engine and any network connection.
 *
 * Please note that the gSOAP engine has no garbage collector. Memory cleanup must be
 * performed by the application programer when it fits its needs best. Please read the gSAOP documentation
 * (provided in this distribution) to learn more on this subject.
 *
 * @param[in] soapenv The SOAP environment to cleanup
 *
 * @return void Should always succeed
 *
 */
static inline void mx_soap_clean(struct soap* soapenv)
{
	soap_destroy(soapenv); // delete deserialized class instances (for C++ only)
	soap_end(soapenv); // remove deserialized data and clean up
	soap_done(soapenv); // detach the gSOAP environment
}

//-----------------------------------------------------------------------------------------
/** if expression is false, prints the error message associated to the current SOAP state to standard error and terminates the program
 *  (i.e., compares equal to zero).
 *
 * @param[in] flag a boolean value indicating an error condition when false
 * @param[in] soapenv The SOAP environment to cleanup
 *
 * @return void
 *
*/
#define assert_gsoap(__soapenv, __fctcall) __assert_gsoap(__soapenv, SOAP_OK==__fctcall,__FILE__,__FUNCTION__,__LINE__)
void __assert_gsoap(struct soap* soapenv, bool flag, const char * _file, const char* _function, int _line);
//-----------------------------------------------------------------------------------------
#define assert_soap_command(__soapenv, __Response) __assert_soap_command(__soapenv, __Response,__FILE__,__FUNCTION__,__LINE__)
void __assert_soap_command(struct soap * soapenv, struct DefaultResponse* Response, const char * _file, const char* _function, int _line);

//-----------------------------------------------------------------------------------------
/** Initialise a SOAP environment.
 *
 * @param[in] soapenv A previously dynamically or on-stack allocated SOAP environment.
 *
 * @return void
 *
 * This function sets some non-default configuration on the underlying TCP/IP socket.
 * These are:
 * - KEEPALIVE
     "This option controls whether the underlying protocol should
     periodically transmit messages on a connected socket.  If the peer
     fails to respond to these messages, the connection is considered
     broken."
 * - TIMEOUT
 * 	 The transmit, receive and connection timeout are set to one seconds.
 *
 */
void mx_soap_init(struct soap * soapenv);
//-----------------------------------------------------------------------------------------
/** parses the string returned by soap_call_MXCommon__GetModuleType() and gives the number of channels the module has.
 *
 * Conventionaly, the begin of a type string is of the form "MX3701-XX-K" where XX is the number of channels
 * Legal values for XX are: "4","8","16"
 *
 * @param[in] str the string to parse
 * @param[in,out] number_of_channels number of channels
 *
 * @return -1 on failure, 0 on success
 *
 * @note The "str" string is assumed to be NULL terminated
 * @note The "str" string may be modified by this function
 *
 */
int parse_number_of_channels (char* str, unsigned int * number_of_channels);

//-----------------------------------------------------------------------------------------
/** Connect to the dataserver of the module.
 *
 * The dataserver is a TCP/IP server whose role is to provide acquired data to connected client.
 * It has been optimized for thoughput and low latency.
 *
 * @param[in] name A symbolic ("mxmodule.mycorp.com") or dotted ("127.0.0.1") network address
 *
 * @return The file descriptor of a connected socket.
 *
 * @note If a failure happens, a error message is printed out and the application terminated.
 */
int datasrv_connect(const char* name);

//-----------------------------------------------------------------------------------------

/** Display a timeval struct in human string
 *
 * Convert and save date and time element
 * of a timeval struct in comprehensible outputs:
 * day/month/years hour:min.:sec.:msec.:usec.
 */
char *DisplayTimeStamp (unsigned long TimeStampSec, unsigned long TimeStampUSec);

#endif // __MX_UTILS_H__
