using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace Diagnostic
{
    public partial class MainForm : Form
    {
        public MainForm()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Occurs the first time the form is loaded
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_Load(object sender, EventArgs e)
        {
            // Load the parameters
            this.ipTextBox.Text = Properties.Settings.Default.ip;
        }

        /// <summary>
        /// Occurs when the form is being closed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            // Save the parameters
            Properties.Settings.Default.ip = this.ipTextBox.Text;
            Properties.Settings.Default.Save();
        }

        /// <summary>
        /// Click on start
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void startButton_Click(object sender, EventArgs e)
        {
            // Update graphics
            this.startButton.Enabled = false;
            this.stopButton.Enabled = true;
            this.statusToolStripProgressBar.Visible = true;

            // Run worker
            monitorBackgroundWorker.RunWorkerAsync(this.ipTextBox.Text);
        }

        /// <summary>
        /// Click on stop
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void stopButton_Click(object sender, EventArgs e)
        {
            monitorBackgroundWorker.CancelAsync();
        }

        /// <summary>
        /// Add a log
        /// </summary>
        /// <param name="log"></param>
        private void AddLog(String log)
        {
            this.logTextBox.Text += DateTime.Now.ToString() + "\t" + log + Environment.NewLine;
            this.logTextBox.SelectionStart = this.logTextBox.Text.Length;
            this.logTextBox.ScrollToCaret();
        }

        /// <summary>
        /// Monitor the diagnostic
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void monitorBackgroundWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            BackgroundWorker worker = (BackgroundWorker)sender;
            string ip = (string)e.Argument;
            MSXE3701 mx = new MSXE3701(ip);
            string error = string.Empty;
            int retval = 0;

            worker.ReportProgress(1, "Monitor started.");

            // Endless loop
            for (; ; )
            {
                ulong powerSupply = 0;
                ulong shortCircuit = 0;

                // Cancel
                if (worker.CancellationPending)
                {
                    worker.ReportProgress(1, "Monitor stopped.");
                    return;
                }

                // Get power supply state
                if ((retval = mx.ExtDigitalIOTestOutputsPowerSupply(0, out powerSupply, ref error)) != 0)
                {
                    worker.ReportProgress(1, error);
                    return;
                }

                // Get short-circuit state
                if ((retval = mx.ExtDigitalIOTestOutputsShortCircuit(0, out shortCircuit, ref error)) != 0)
                {
                    worker.ReportProgress(1, error);
                    return;
                }

                // Report
                worker.ReportProgress(99, new WorkerReport(powerSupply, shortCircuit));

                // Sleep
                System.Threading.Thread.Sleep(500);
            }
        }

        /// <summary>
        /// Done
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void monitorBackgroundWorker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            this.statusToolStripProgressBar.Visible = false;
            this.startButton.Enabled = true;
            this.stopButton.Enabled = false;
        }

        /// <summary>
        /// Worker report information
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void monitorBackgroundWorker_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            // Text message ?
            if (e.ProgressPercentage == 1)
            {
                this.AddLog((string)e.UserState);
            }
            // Report ?
            else if (e.ProgressPercentage == 99)
            {
                WorkerReport report = (WorkerReport)e.UserState;
                // Power supply
                if (report.powerSupply == 1)
                {
                    this.powerSupplyTextBox.Text = "No power supply detected.";
                    this.powerSupplyPanel.BackColor = Color.Red;
                }
                else
                {
                    this.powerSupplyTextBox.Text = "Ok.";
                    this.powerSupplyPanel.BackColor = Color.Chartreuse;
                }
                // Short circuit
                if (report.shortCircuit == 1)
                {
                    this.shortCircuitTextBox.Text = "Short circuit detected.";
                    this.shortCircuitPanel.BackColor = Color.Red;
                }
                else
                {
                    this.shortCircuitTextBox.Text = "Ok.";
                    this.shortCircuitPanel.BackColor = Color.Chartreuse;
                }
            }
        }
    }
}