using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace Information
{
    public partial class MainForm : Form
    {
        /// <summary>
        /// Constructor
        /// </summary>
        public MainForm()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Occurs the first time the form is loaded
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_Load(object sender, EventArgs e)
        {
            // Load the parameters
            this.ipTextBox.Text = Properties.Settings.Default.ip;
        }

        /// <summary>
        /// Occurs when the form is being closed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            // Save the parameters
            Properties.Settings.Default.ip = this.ipTextBox.Text;
            Properties.Settings.Default.Save();
        }

        /// <summary>
        /// Add a log
        /// </summary>
        /// <param name="log"></param>
        private void AddLog(String log)
        {
            this.logTextBox.Text += DateTime.Now.ToString() + "\t" + log + Environment.NewLine;
            this.logTextBox.SelectionStart = this.logTextBox.Text.Length;
            this.logTextBox.ScrollToCaret();
        }

        /// <summary>
        /// Get the informations
        /// Since the operation may take time, we do the action in a background worker
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void getInfoButton_Click(object sender, EventArgs e)
        {
            this.statusToolStripProgressBar.Visible = true;
            this.getInfoButton.Enabled = false;

            // Start the background worker
            getInfoBackgroundWorker.RunWorkerAsync(this.ipTextBox.Text);
        }

        /// <summary>
        /// Get the informations
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void getInfoBackgroundWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            BackgroundWorker worker = (BackgroundWorker)sender;
            string ip = (string)e.Argument;
            MSXE3701 mx = new MSXE3701(ip);
            int retval = 0;
            string error = string.Empty;
            ulong channelNumber = 0;
            ulong portNumber = 0;
            ulong[] channelsPerPort = null;
            ulong[] portsDirection = null;

            // Get the number of channels
            if ((retval = mx.ExtDigitalIOGetNumberOfChannels(out channelNumber, ref error)) != 0)
            {
                worker.ReportProgress(1, error);
                return;
            }

            // Get the number of ports
            if ((retval = mx.ExtDigitalIOGetNumberOfPorts(out portNumber, ref error)) != 0)
            {
                worker.ReportProgress(1, error);
                return;
            }

            // Allocate space
            channelsPerPort = new ulong[portNumber];
            portsDirection = new ulong[portNumber];

            // Get information concerning the port
            for (ulong i = 0; i < portNumber; i++)
            {
                ulong ch = 0;
                ulong dir = 0;

                // Number of channels per port
                if ((retval = mx.ExtDigitalIOGetNumberOfChannelsPerPort(i, out ch, ref error)) != 0)
                {
                    worker.ReportProgress(1, error);
                    return;
                }
                channelsPerPort[i] = ch;

                // Port direction
                if ((retval = mx.ExtDigitalIOGetPortDirections(i, out dir, ref error)) != 0)
                {
                    worker.ReportProgress(1, error);
                    return;
                }
                portsDirection[i] = dir;
            }

            // Create report object and return it
            WorkerReport report = new WorkerReport(channelNumber, portNumber, channelsPerPort, portsDirection);
            worker.ReportProgress(99, report);
        }

        /// <summary>
        /// Worker reports an information
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void getInfoBackgroundWorker_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            // Text message ?
            if (e.ProgressPercentage == 1)
            {
                this.AddLog((string)e.UserState);
            }
            // Report ?
            else if (e.ProgressPercentage == 99)
            {
                WorkerReport report = (WorkerReport)e.UserState;
                this.channelNumberTextBox.Text = report.channelNumber.ToString();
                this.portNumberTextBox.Text = report.portNumber.ToString();

                // Clear
                this.channelsPerPortListView.Items.Clear();
                this.portsDirectionListView.Items.Clear();

                // Add new
                for (ulong i = 0; i < report.portNumber; i++)
                {
                    this.channelsPerPortListView.Items.Add(new ListViewItem(new string[] { i.ToString(), report.channelsPerPort[i].ToString() }));
                    this.portsDirectionListView.Items.Add(new ListViewItem(new string[] { i.ToString(), "0x" + report.portsDirection[i].ToString("X") }));
                }
            }
        }

        /// <summary>
        /// Worker is over
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void getInfoBackgroundWorker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            // Update the graphics
            this.getInfoButton.Enabled = true;
            this.statusToolStripProgressBar.Visible = false;
        }
    }
}