using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;

namespace ReadWrite
{
    public partial class MainForm : Form
    {
        /// <summary>
        /// The MSX-Exxxx system
        /// </summary>
        private MSXE3701 mx = null;

        /// <summary>
        /// Store the number of ports
        /// </summary>
        private ulong portNumber = 0;

        /// <summary>
        /// Store the number of channels
        /// </summary>
        private ulong channelNumber = 0;

        /// <summary>
        /// Store the number of channels per port
        /// </summary>
        private ulong[] channelsPerPort = null;

        /// <summary>
        /// Store the direction of the channels
        /// </summary>
        private ulong[] portDirection = null;

        public MainForm()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Occurs the first time the form is loaded
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_Load(object sender, EventArgs e)
        {
            // Load the parameters
            this.ipTextBox.Text = Properties.Settings.Default.ip;

            // Default
            this.readAccessComboBox.SelectedIndex = 0;
            this.stateComboBox.SelectedIndex = 0;
        }

        /// <summary>
        /// Occurs when the form is being closed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            // Save the parameters
            Properties.Settings.Default.ip = this.ipTextBox.Text;
            Properties.Settings.Default.Save();
        }

        /// <summary>
        /// Add a log
        /// </summary>
        /// <param name="log"></param>
        private void AddLog(String log)
        {
            this.logTextBox.Text += DateTime.Now.ToString() + "\t" + log + Environment.NewLine;
            this.logTextBox.SelectionStart = this.logTextBox.Text.Length;
            this.logTextBox.ScrollToCaret();
        }

        /// <summary>
        /// Init and start
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void initAndStartButton_Click(object sender, EventArgs e)
        {
            int retval = 0;
            string error = string.Empty;

            // Create MSX-E system
            this.mx = new MSXE3701(this.ipTextBox.Text);

            // Get the number of ports
            if ((retval = this.mx.ExtDigitalIOGetNumberOfPorts(out portNumber, ref error)) != 0)
            {
                this.AddLog(error);
                return;
            }

            // Get the number of channels
            if ((retval = this.mx.ExtDigitalIOGetNumberOfChannels(out channelNumber, ref error)) != 0)
            {
                this.AddLog(error);
                return;
            }

            // Allocate some space
            this.channelsPerPort = new ulong[portNumber];
            this.portDirection = new ulong[portNumber];

            // For each port, get the information
            for (ulong i = 0; i < portNumber; i++)
            {
                ulong ch = 0;
                ulong dir = 0;

                // Number of channels
                if ((retval = this.mx.ExtDigitalIOGetNumberOfChannelsPerPort(i, out ch, ref error)) != 0)
                {
                    this.AddLog(error);
                    return;
                }

                // Direction of the channels
                if ((retval = this.mx.ExtDigitalIOGetPortDirections(i, out dir, ref error)) != 0)
                {
                    this.AddLog(error);
                    return;
                }

                // Save
                this.channelsPerPort[i] = ch;
                this.portDirection[i] = dir;
            }

            // Initialise the outputs port combo box
            string[] portSelection = new string[portNumber];
            for (ulong i = 0; i < portNumber; i++)
            {
                portSelection[i] = i.ToString();
            }
            this.portComboBox.DataSource = portSelection;

            // Initialise the outputs channel combo box
            string[] channelSelection = new string[channelNumber];
            for (ulong i = 0; i < channelNumber; i++)
            {
                channelSelection[i] = i.ToString();
            }
            this.channelComboBox.DataSource = channelSelection;

            // Initialise the list
            this.monitorListView.Items.Clear();
            if (readAccessComboBox.SelectedIndex == 0)
            {
                for (ulong i = 0; i < portNumber; i++)
                {
                    for (ulong j = 0; j < channelsPerPort[i]; j++)
                    {
                        this.monitorListView.Items.Add(new ListViewItem(new string[] { i.ToString(),
                                j.ToString(),
                                ( ((portDirection[i] >> (int)j) & 1) == 1) ? "Output" : "Input",
                                "-" }));
                    }
                }
            }
            else
            {
                for (ulong i = 0; i < channelNumber; i++)
                {
                    this.monitorListView.Items.Add(new ListViewItem(new string[] { "-",
                                i.ToString(),
                                "-",
                                "-" }));
                }
            }

            // Start monitor worker
            monitorBackgroundWorker.RunWorkerAsync(new WorkerParams(portNumber, channelNumber, readAccessComboBox.SelectedIndex, mx));

            // Update graphics
            this.monitorOptionGroupBox.Enabled = false;
            this.initAndStartButton.Enabled = false;
            this.stopButton.Enabled = true;
            this.statusToolStripProgressBar.Visible = true;
            this.outputsGroupBox.Enabled = true;
        }

        /// <summary>
        /// Monitor work
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void monitorBackgroundWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            BackgroundWorker worker = (BackgroundWorker)sender;
            WorkerParams parameters = (WorkerParams)e.Argument;
            int retval = 0;
            string error = string.Empty;
            Stopwatch sw = new Stopwatch();

            // Endless loop
            for (; ; )
            {
                // Should stop ? 
                if (worker.CancellationPending)
                    return;

                // By port ?
                if (parameters.method == 0)
                {
                    ulong[] portState = new ulong[parameters.portNumber];
                    sw.Start();
                    for (ulong i = 0; i < parameters.portNumber; i++)
                    {
                        ulong state = 0;
                        // Read the state
                        if ((retval = parameters.mx.ExtDigitalIOReadPort(i, out state, ref error)) != 0)
                        {
                            worker.ReportProgress(1, error);
                            return;
                        }
                        // Save
                        portState[i] = state;
                    }
                    sw.Stop();
                    
                    // Report change
                    worker.ReportProgress(10, portState);

                    // Report time change
                    worker.ReportProgress(2, sw.Elapsed.TotalMilliseconds);
                    sw.Reset();
                }
                // By channel
                else if (parameters.method == 1)
                {
                    ulong[] channelState = new ulong[parameters.channelNumber];
                    sw.Start();
                    for (ulong i = 0; i < parameters.channelNumber; i++)
                    {
                        ulong state = 0;
                        // Read the state
                        if ((retval = parameters.mx.ExtDigitalIOReadChannel(i, out state, ref error)) != 0)
                        {
                            worker.ReportProgress(1, error);
                            return;
                        }
                        // Save
                        channelState[i] = state;
                    }
                    sw.Stop();

                    // Report change
                    worker.ReportProgress(20, channelState);

                    // Report time change
                    worker.ReportProgress(2, sw.Elapsed.TotalMilliseconds);
                    sw.Reset();
                }

                System.Threading.Thread.Sleep(500);
            }
        }

        /// <summary>
        /// Click on stop
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void stopButton_Click(object sender, EventArgs e)
        {
            this.stopButton.Enabled = false;
            monitorBackgroundWorker.CancelAsync();
        }

        /// <summary>
        /// Monitor is over
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void monitorBackgroundWorker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            this.initAndStartButton.Enabled = true;
            this.monitorOptionGroupBox.Enabled = true;
            this.statusToolStripProgressBar.Visible = false;
            this.outputsGroupBox.Enabled = false;
        }

        /// <summary>
        /// Worker report information
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void monitorBackgroundWorker_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            // Text message ?
            if (e.ProgressPercentage == 1)
            {
                this.AddLog((string)e.UserState);
            }
            // Elapsed time
            if (e.ProgressPercentage == 2)
            {
                double elapsed = (double)e.UserState;
                this.timeNeededTextBox.Text = elapsed.ToString("F1");
            }
            // Port message
            if (e.ProgressPercentage == 10)
            {
                ulong [] portState = (ulong[])e.UserState;
                int index = 0;
                // Update values
                for (ulong i = 0; i < portNumber; i++)
                {
                    for (ulong j = 0; j < channelsPerPort[i]; j++)
                    {
                        this.monitorListView.Items[index].SubItems[3].Text = (((portState[i] >> (int)j) & 1) == 1) ? "High" : "Low";
                        if (((portState[i] >> (int)j) & 1) == 1)
                        {
                            this.monitorListView.Items[index].ForeColor = Color.Red;
                        }
                        else
                        {
                            this.monitorListView.Items[index].ForeColor = Color.Green;
                        }
                        ++index;
                    }
                }
            }
            // Channel message
            if (e.ProgressPercentage == 20)
            {
                ulong[] channelState = (ulong[])e.UserState;
                // Update values
                for (ulong i = 0; i < channelNumber; i++)
                {
                    this.monitorListView.Items[(int)i].SubItems[3].Text = (channelState[i] == 1) ? "High" : "Low";
                    if (channelState[i] == 1)
                    {
                        this.monitorListView.Items[(int)i].ForeColor = Color.Red;
                    }
                    else
                    {
                        this.monitorListView.Items[(int)i].ForeColor = Color.Green;
                    }
                }
            }
        }

        /// <summary>
        /// Update with current value
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void updatePortButton_Click(object sender, EventArgs e)
        {
            int retval = 0;
            string error = string.Empty;
            
            // Check
            if (portComboBox.SelectedIndex < 0)
                return;

            if (this.mx == null)
                return;

            ulong port = (ulong)portComboBox.SelectedIndex;
            ulong state = 0;

            // Get port value
            if ((retval = this.mx.ExtDigitalIOReadPort(port, out state, ref error)) != 0)
            {
                this.AddLog(error);
                return;
            }

            // Display
            this.portValueTextBox.Text = "0x" + state.ToString("X");
        }

        /// <summary>
        /// Parse an unsigned number
        /// </summary>
        /// <param name="number"></param>
        /// <param name="value"></param>
        /// <returns></returns>
        private bool ParseNumber(string number, out ulong value)
        {
            try
            {
                if ((number.Length > 2) && (number[0] == '0') && (number[1] == 'x'))
                {
                    value = Convert.ToUInt64(number.Substring(2), 16);
                }
                else
                {
                    value = Convert.ToUInt64(number);
                }
            }
            catch (Exception)
            {
                value = 0;
                return false;
            }
            return true;
        }

        /// <summary>
        /// Click on the write port button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void writePortButton_Click(object sender, EventArgs e)
        {
            int retval = 0;
            string error = string.Empty;

            // Check
            if (portComboBox.SelectedIndex < 0)
                return;

            if (this.mx == null)
                return;

            ulong port = (ulong)portComboBox.SelectedIndex;
            ulong state = 0;
            if (ParseNumber(this.portValueTextBox.Text, out state) == false)
            {
                this.AddLog("Wrong port state.");
                return;
            }

            // Set the value
            if ((retval = this.mx.ExtDigitalIOWritePort(port, state, ref error)) != 0)
            {
                this.AddLog(error);
                return;
            }
        }

        /// <summary>
        /// Update channel with current value
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void updateChannelValueButton_Click(object sender, EventArgs e)
        {
            int retval = 0;
            string error = string.Empty;

            // Check
            if (channelComboBox.SelectedIndex < 0)
                return;

            if (this.mx == null)
                return;

            ulong channel = (ulong)channelComboBox.SelectedIndex;
            ulong state = 0;

            // Get channel value
            if ((retval = this.mx.ExtDigitalIOReadChannel(channel,out state, ref error))!= 0)
            {
                this.AddLog(error);
                return;
            }

            // Display
            if ( state == 0 )
                this.stateComboBox.SelectedIndex = 0;
            else
                this.stateComboBox.SelectedIndex = 1;
        }

        /// <summary>
        /// Set value of a channel
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void writeChannelButton_Click(object sender, EventArgs e)
        {
            int retval = 0;
            string error = string.Empty;

            // Check
            if (channelComboBox.SelectedIndex < 0)
                return;

            if ( stateComboBox.SelectedIndex < 0)
                return;

            if (this.mx == null)
                return;

            ulong channel = (ulong)channelComboBox.SelectedIndex;
            ulong state = (ulong)stateComboBox.SelectedIndex;

            // Set the value
            if ((retval = this.mx.ExtDigitalIOWriteChannel(channel,state, ref error)) != 0)
            {
                this.AddLog(error);
                return;
            }
        }
    }
}